@@ -6,7 +6,6 @@ import tempfile
 import uuid
 
 import msgpack
-
 from kodexa.model import Document, ContentNode, SourceMetadata
 from kodexa.model.model import ContentClassification, DocumentMetadata, ContentFeature
 
@@ -65,6 +64,26 @@ class SqliteDocumentPersistence(object):
         self.cursor.execute("pragma temp_store = memory")
         self.cursor.execute("pragma mmap_size = 30000000000")
 
+    def update_features(self,node):
+
+        next_feature_id = self.get_max_feature_id()
+        all_features = []
+        for feature in node.get_features():
+            binary_value = sqlite3.Binary(msgpack.packb(feature.value, use_bin_type=True))
+
+            tag_uuid = None
+            if feature.feature_type == 'tag' and 'uuid' in feature.value[0]:
+                tag_uuid = feature.value[0]['uuid']
+
+            all_features.append(
+                [next_feature_id, node.uuid, self.get_feature_type_id(feature),
+                 binary_value, feature.single, tag_uuid])
+
+            next_feature_id = next_feature_id + 1
+
+        self.cursor.execute("DELETE FROM ft where cn_id=?", [node.uuid])
+        self.cursor.executemany(FEATURE_INSERT, all_features)
+
     def update_node(self, node):
         self.cursor.execute('update cn set idx=?, pid=? where id=?',
                             [node.index, node._parent_uuid,
@@ -79,14 +98,14 @@ class SqliteDocumentPersistence(object):
             if node_type == "*":
                 query = """
                             with recursive
-                            parent_node(id, pid, nt, idx) AS (
-                                VALUES (?,?,?,?)
+                            parent_node(id, pid, nt, idx, path) AS (
+                                VALUES (?,?,?,?,?)
                                 UNION ALL
-                                SELECT cns.id, cns.pid, cns.nt, cns.idx 
+                                SELECT cns.id, cns.pid, cns.nt, cns.idx, parent_node.path || substr('0000000' || cns.idx, -6, 6) 
                                 FROM cn cns, parent_node
                                 WHERE parent_node.id = cns.pid  
                             )
-                            SELECT id, pid, nt, idx from parent_node order by parent_node.idx, idx
+                            SELECT id, pid, nt, idx, path from parent_node order by path
                             """
 
                 try:
@@ -95,20 +114,20 @@ class SqliteDocumentPersistence(object):
                                                    parent_node.get_parent().uuid if parent_node.get_parent() else None,
                                                    next(key for key, value in self.node_types.items() if
                                                         value == parent_node.get_node_type()),
-                                                   parent_node.index]).fetchall()
+                                                   parent_node.index, f"{parent_node.index}".zfill(6)]).fetchall()
                 except StopIteration:
                     return []
             else:
                 query = """
                                 with recursive
-                                parent_node(id, pid, nt, idx) AS (
-                                    VALUES (?,?,?,?)
+                                parent_node(id, pid, nt, idx, path) AS (
+                                    VALUES (?,?,?,?,?)
                                     UNION ALL
-                                    SELECT cns.id, cns.pid, cns.nt, cns.idx 
+                                    SELECT cns.id, cns.pid, cns.nt, cns.idx, parent_node.path || substr('000000' || cns.idx, -6, 6) 
                                     FROM cn cns, parent_node
                                     WHERE parent_node.id = cns.pid  
                                 )
-                                SELECT id, pid, nt, idx from parent_node where nt=? order by parent_node.idx, idx
+                                SELECT id, pid, nt, idx, path from parent_node where nt=? order by path
                                 """
 
                 try:
@@ -118,6 +137,7 @@ class SqliteDocumentPersistence(object):
                                                    next(key for key, value in self.node_types.items() if
                                                         value == parent_node.get_node_type()),
                                                    parent_node.index,
+                                                   f"{parent_node.index}".zfill(6),
                                                    next(key for key, value in self.node_types.items() if
                                                         value == node_type)]).fetchall()
                 except StopIteration:
@@ -382,7 +402,7 @@ class SqliteDocumentPersistence(object):
 
         parent = self.cursor.execute("select pid from cn where id = ?", [content_node.uuid]).fetchone()
         if parent:
-            return self.__get_node(parent[0])
+            return self.get_node(parent[0])
         else:
             return None
 
@@ -497,7 +517,8 @@ class SimpleObjectCache(object):
     def remove_obj(self, obj):
         if obj.uuid in self.objs:
             self.objs.pop(obj.uuid)
-            self.dirty_objs.remove(obj.uuid)
+            if obj.uuid in self.dirty_objs:
+                self.dirty_objs.remove(obj.uuid)
 
     def get_dirty_objs(self):
         results = []
@@ -551,7 +572,6 @@ class PersistenceManager(object):
         all_features = []
         node_id_with_features = []
 
-
         logger.info("Merging cache to persistance")
         dirty_nodes = self.node_cache.get_dirty_objs()
 
@@ -696,6 +716,9 @@ class PersistenceManager(object):
         return self.child_cache[node.uuid]
 
     def update_node(self, node):
+        # We need to also update the parent
+        self.node_parent_cache[node.uuid] = node._parent_uuid
+
         self._underlying_persistence.update_node(node)
 
     def update_content_parts(self, node, content_parts):
@@ -716,6 +739,7 @@ class PersistenceManager(object):
     def remove_feature(self, node, feature_type, name):
 
         features = self.get_features(node)
+        self._underlying_persistence.remove_feature(node, feature_type, name)
         new_features = [i for i in features if not (i.feature_type == feature_type and i.name == name)]
         self.feature_cache[node.uuid] = new_features
         self.node_cache.add_obj(node)
@@ -733,6 +757,6 @@ class PersistenceManager(object):
         if node.uuid not in self.feature_cache:
             features = self._underlying_persistence.get_features(node)
             self.feature_cache[node.uuid] = features
-            self.node_cache.add_obj(node)
+
         self.node_cache.add_obj(node)
-        self.feature_cache[node.uuid].append(feature)
+        self.feature_cache[node.uuid].append(feature)
\ No newline at end of file
